/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2007 David Smith (catfish.man@gmail.com)
 * Copyright (C) 2003-2013 Apple Inc. All rights reserved.
 * Copyright (C) Research In Motion Limited 2010. All rights reserved.
 * Copyright (C) 2013 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_LAYOUT_TEXT_RUN_CONSTRUCTOR_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_LAYOUT_TEXT_RUN_CONSTRUCTOR_H_

#include "third_party/blink/renderer/platform/text/text_direction.h"
#include "third_party/blink/renderer/platform/text/text_run.h"
#include "third_party/blink/renderer/platform/wtf/text/unicode.h"
#include "third_party/blink/renderer/platform/wtf/text/wtf_string.h"

namespace blink {

class Font;
class ComputedStyle;
class LayoutText;
class LineLayoutText;

enum TextRunFlag {
  kDefaultTextRunFlags = 0,
  kRespectDirection = 1 << 0,
  kRespectDirectionOverride = 1 << 1
};

typedef unsigned TextRunFlags;

// Direction resolved from string value.
TextRun ConstructTextRun(const Font&,
                         const String&,
                         const ComputedStyle&,
                         TextRunFlags = kDefaultTextRunFlags);
TextRun ConstructTextRun(const Font&,
                         const LineLayoutText,
                         unsigned offset,
                         unsigned length,
                         const ComputedStyle&);

// Explicit direction.
TextRun ConstructTextRun(const Font&,
                         const String&,
                         const ComputedStyle&,
                         TextDirection,
                         TextRunFlags = kDefaultTextRunFlags);
TextRun ConstructTextRun(const Font&,
                         const LayoutText*,
                         unsigned offset,
                         unsigned length,
                         const ComputedStyle&,
                         TextDirection);
TextRun ConstructTextRun(const Font&,
                         const LChar*,
                         int length,
                         const ComputedStyle&,
                         TextDirection);
TextRun ConstructTextRun(const Font&,
                         const UChar*,
                         int length,
                         const ComputedStyle&,
                         TextDirection);

}  // namespace blink

#endif
